@@ -12,7 +12,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkFrame.c,v 1.3 1999/04/16 01:51:14 stanton Exp $
+ * RCS: @(#) $Id: tkFrame.c,v 1.4 1999/08/10 05:06:01 jingham Exp $
  */
 
 #include "default.h"
@@ -191,7 +191,7 @@ static Tk_ConfigSpec configSpecs[] = {
  */
 
 static int		ConfigureFrame _ANSI_ARGS_((Tcl_Interp *interp,
-			    Frame *framePtr, int argc, char **argv,
+			    Frame *framePtr, int objc, Tcl_Obj *CONST objv[],
 			    int flags));
 static void		DestroyFrame _ANSI_ARGS_((char *memPtr));
 static void		DisplayFrame _ANSI_ARGS_((ClientData clientData));
@@ -199,14 +199,14 @@ static void		FrameCmdDeletedProc _ANSI_ARGS_((
 			    ClientData clientData));
 static void		FrameEventProc _ANSI_ARGS_((ClientData clientData,
 			    XEvent *eventPtr));
-static int		FrameWidgetCmd _ANSI_ARGS_((ClientData clientData,
-			    Tcl_Interp *interp, int argc, char **argv));
+static int		FrameWidgetObjCmd _ANSI_ARGS_((ClientData clientData,
+			    Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]));
 static void		MapFrame _ANSI_ARGS_((ClientData clientData));
 
 /*
  *--------------------------------------------------------------
  *
- * Tk_FrameCmd, Tk_ToplevelCmd --
+ * Tk_FrameObjCmd, Tk_ToplevelObjCmd --
  *
  *	These procedures are invoked to process the "frame" and
  *	"toplevel" Tcl commands.  See the user documentation for
@@ -223,25 +223,25 @@ static void		MapFrame _ANSI_ARGS_((ClientData clientData));
  */
 
 int
-Tk_FrameCmd(clientData, interp, argc, argv)
+Tk_FrameObjCmd(clientData, interp, objc, objv)
     ClientData clientData;	/* Main window associated with
 				 * interpreter. */
     Tcl_Interp *interp;		/* Current interpreter. */
-    int argc;			/* Number of arguments. */
-    char **argv;		/* Argument strings. */
+    int objc;			/* Number of arguments. */
+    Tcl_Obj *CONST objv[];	/* Argument objects. */
 {
-    return TkCreateFrame(clientData, interp, argc, argv, 0, (char *) NULL);
+    return TkCreateFrame(clientData, interp, objc, objv, 0, (char *) NULL);
 }
 
 int
-Tk_ToplevelCmd(clientData, interp, argc, argv)
+Tk_ToplevelObjCmd(clientData, interp, objc, objv)
     ClientData clientData;	/* Main window associated with
 				 * interpreter. */
     Tcl_Interp *interp;		/* Current interpreter. */
-    int argc;			/* Number of arguments. */
-    char **argv;		/* Argument strings. */
+    int objc;			/* Number of arguments. */
+    Tcl_Obj *CONST objv[];	/* Argument objects. */
 {
-    return TkCreateFrame(clientData, interp, argc, argv, 1, (char *) NULL);
+    return TkCreateFrame(clientData, interp, objc, objv, 1, (char *) NULL);
 }
 
 /*
@@ -264,13 +264,13 @@ Tk_ToplevelCmd(clientData, interp, argc, argv)
  */
 
 int
-TkCreateFrame(clientData, interp, argc, argv, toplevel, appName)
+TkCreateFrame(clientData, interp, objc, objv, toplevel, appName)
     ClientData clientData;	/* Main window associated with interpreter.
 				 * If we're called by Tk_Init to create a
 				 * new application, then this is NULL. */
     Tcl_Interp *interp;		/* Current interpreter. */
-    int argc;			/* Number of arguments. */
-    char **argv;		/* Argument strings. */
+    int objc;			/* Number of arguments. */
+    Tcl_Obj *CONST objv[];	/* Argument objects. */
     int toplevel;		/* Non-zero means create a toplevel window,
 				 * zero means create a frame. */
     char *appName;		/* Should only be non-NULL if clientData is
@@ -286,9 +286,8 @@ TkCreateFrame(clientData, interp, argc, argv, toplevel, appName)
     Colormap colormap;
     Visual *visual;
 
-    if (argc < 2) {
-	Tcl_AppendResult(interp, "wrong # args: should be \"",
-		argv[0], " pathName ?options?\"", (char *) NULL);
+    if (objc < 2) {
+	Tcl_WrongNumArgs(interp, 1, objv, "pathName ?options?");
 	return TCL_ERROR;
     }
 
@@ -301,28 +300,27 @@ TkCreateFrame(clientData, interp, argc, argv, toplevel, appName)
 
     className = colormapName = screenName = visualName = useOption = NULL;
     colormap = None;
-    for (i = 2; i < argc; i += 2) {
-	arg = argv[i];
-	length = strlen(arg);
+    for (i = 2; i < objc; i += 2) {
+	arg = Tcl_GetStringFromObj(objv[i], &length);
 	if (length < 2) {
 	    continue;
 	}
 	c = arg[1];
-	if ((c == 'c') && (strncmp(arg, "-class", strlen(arg)) == 0)
+	if ((c == 'c') && (strncmp(arg, "-class", length) == 0)
 		&& (length >= 3)) {
-	    className = argv[i+1];
+	    className = Tcl_GetString(objv[i+1]);
 	} else if ((c == 'c')
-		&& (strncmp(arg, "-colormap", strlen(arg)) == 0)) {
-	    colormapName = argv[i+1];
+		&& (strncmp(arg, "-colormap", length) == 0)) {
+	    colormapName = Tcl_GetString(objv[i+1]);
 	} else if ((c == 's') && toplevel
-		&& (strncmp(arg, "-screen", strlen(arg)) == 0)) {
-	    screenName = argv[i+1];
+		&& (strncmp(arg, "-screen", length) == 0)) {
+	    screenName = Tcl_GetString(objv[i+1]);
 	} else if ((c == 'u') && toplevel
-		&& (strncmp(arg, "-use", strlen(arg)) == 0)) {
-	    useOption = argv[i+1];
+		&& (strncmp(arg, "-use", length) == 0)) {
+	    useOption = Tcl_GetString(objv[i+1]);
 	} else if ((c == 'v')
-		&& (strncmp(arg, "-visual", strlen(arg)) == 0)) {
-	    visualName = argv[i+1];
+		&& (strncmp(arg, "-visual", length) == 0)) {
+	    visualName = Tcl_GetString(objv[i+1]);
 	}
     }
 
@@ -345,7 +343,7 @@ TkCreateFrame(clientData, interp, argc, argv, toplevel, appName)
 	screenName = (toplevel) ? "" : NULL;
     }
     if (tkwin != NULL) {
-	new = Tk_CreateWindowFromPath(interp, tkwin, argv[1], screenName);
+	new = Tk_CreateWindowFromPath(interp, tkwin, Tcl_GetString(objv[1]), screenName);
     } else {
 	/*
 	 * We were called from Tk_Init;  create a new application.
@@ -416,8 +414,8 @@ TkCreateFrame(clientData, interp, argc, argv, toplevel, appName)
     framePtr->tkwin = new;
     framePtr->display = Tk_Display(new);
     framePtr->interp = interp;
-    framePtr->widgetCmd = Tcl_CreateCommand(interp,
-	    Tk_PathName(new), FrameWidgetCmd,
+    framePtr->widgetCmd = Tcl_CreateObjCommand(interp,
+	    Tk_PathName(new), FrameWidgetObjCmd,
 	    (ClientData) framePtr, FrameCmdDeletedProc);
     framePtr->className = NULL;
     framePtr->mask = (toplevel) ? TOPLEVEL : FRAME;
@@ -455,7 +453,7 @@ TkCreateFrame(clientData, interp, argc, argv, toplevel, appName)
         mask |= ActivateMask;
     }
     Tk_CreateEventHandler(new, mask, FrameEventProc, (ClientData) framePtr);
-    if (ConfigureFrame(interp, framePtr, argc-2, argv+2, 0) != TCL_OK) {
+    if (ConfigureFrame(interp, framePtr, objc-2, objv+2, 0) != TCL_OK) {
 	goto error;
     }
     if ((framePtr->isContainer)) {
@@ -483,7 +481,7 @@ TkCreateFrame(clientData, interp, argc, argv, toplevel, appName)
 /*
  *--------------------------------------------------------------
  *
- * FrameWidgetCmd --
+ * FrameWidgetObjCmd --
  *
  *	This procedure is invoked to process the Tcl command
  *	that corresponds to a frame widget.  See the user
@@ -499,83 +497,87 @@ TkCreateFrame(clientData, interp, argc, argv, toplevel, appName)
  */
 
 static int
-FrameWidgetCmd(clientData, interp, argc, argv)
+FrameWidgetObjCmd(clientData, interp, objc, objv)
     ClientData clientData;	/* Information about frame widget. */
     Tcl_Interp *interp;		/* Current interpreter. */
-    int argc;			/* Number of arguments. */
-    char **argv;		/* Argument strings. */
+    int objc;			/* Number of arguments. */
+    Tcl_Obj *CONST objv[];	/* Argument objects. */
 {
+    static char *frameOptions[] = {
+	"cget", "configure", (char *) NULL
+    };
+    enum options {
+	FRAME_CGET, FRAME_CONFIGURE
+    };
     register Frame *framePtr = (Frame *) clientData;
-    int result;
+    int result = TCL_OK, index;
     size_t length;
     int c, i;
 
-    if (argc < 2) {
-	Tcl_AppendResult(interp, "wrong # args: should be \"",
-		argv[0], " option ?arg arg ...?\"", (char *) NULL);
+    if (objc < 2) {
+	Tcl_WrongNumArgs(interp, 1, objv, "option ?arg arg ...?");
+	return TCL_ERROR;
+    }
+    if (Tcl_GetIndexFromObj(interp, objv[1], frameOptions, "option", 0,
+	    &index) != TCL_OK) {
 	return TCL_ERROR;
     }
     Tcl_Preserve((ClientData) framePtr);
-    c = argv[1][0];
-    length = strlen(argv[1]);
-    if ((c == 'c') && (strncmp(argv[1], "cget", length) == 0)
-	    && (length >= 2)) {
-	if (argc != 3) {
-	    Tcl_AppendResult(interp, "wrong # args: should be \"",
-		    argv[0], " cget option\"",
-		    (char *) NULL);
+    switch ((enum options) index) {
+      case FRAME_CGET: {
+	if (objc != 3) {
+	    Tcl_WrongNumArgs(interp, 2, objv, "option");
 	    result = TCL_ERROR;
 	    goto done;
 	}
 	result = Tk_ConfigureValue(interp, framePtr->tkwin, configSpecs,
-		(char *) framePtr, argv[2], framePtr->mask);
-    } else if ((c == 'c') && (strncmp(argv[1], "configure", length) == 0)
-	    && (length >= 2)) {
-	if (argc == 2) {
+		(char *) framePtr, Tcl_GetString(objv[2]), framePtr->mask);
+	break;
+      }
+      case FRAME_CONFIGURE: {
+	if (objc == 2) {
 	    result = Tk_ConfigureInfo(interp, framePtr->tkwin, configSpecs,
 		    (char *) framePtr, (char *) NULL, framePtr->mask);
-	} else if (argc == 3) {
+	} else if (objc == 3) {
 	    result = Tk_ConfigureInfo(interp, framePtr->tkwin, configSpecs,
-		    (char *) framePtr, argv[2], framePtr->mask);
+		    (char *) framePtr, Tcl_GetString(objv[2]), framePtr->mask);
 	} else {
 	    /*
 	     * Don't allow the options -class, -colormap, -container,
 	     * -newcmap, -screen, -use, or -visual to be changed.
 	     */
 
-	    for (i = 2; i < argc; i++) {
-		length = strlen(argv[i]);
+	    for (i = 2; i < objc; i++) {
+		char *arg = Tcl_GetStringFromObj(objv[i], &length);
 		if (length < 2) {
 		    continue;
 		}
-		c = argv[i][1];
-		if (((c == 'c') && (strncmp(argv[i], "-class", length) == 0)
+		c = arg[1];
+		if (((c == 'c') && (strncmp(arg, "-class", length) == 0)
 			&& (length >= 2))
 			|| ((c == 'c') && (framePtr->mask == TOPLEVEL)
-			&& (strncmp(argv[i], "-colormap", length) == 0)
+			&& (strncmp(arg, "-colormap", length) == 0)
 			&& (length >= 3))
 			|| ((c == 'c')
-			&& (strncmp(argv[i], "-container", length) == 0)
+			&& (strncmp(arg, "-container", length) == 0)
 			&& (length >= 3))
 			|| ((c == 's') && (framePtr->mask == TOPLEVEL)
-			&& (strncmp(argv[i], "-screen", length) == 0))
+			&& (strncmp(arg, "-screen", length) == 0))
 			|| ((c == 'u') && (framePtr->mask == TOPLEVEL)
-			&& (strncmp(argv[i], "-use", length) == 0))
+			&& (strncmp(arg, "-use", length) == 0))
 			|| ((c == 'v') && (framePtr->mask == TOPLEVEL)
-			&& (strncmp(argv[i], "-visual", length) == 0))) {
-		    Tcl_AppendResult(interp, "can't modify ", argv[i],
+			&& (strncmp(arg, "-visual", length) == 0))) {
+		    Tcl_AppendResult(interp, "can't modify ", arg,
 			    " option after widget is created", (char *) NULL);
 		    result = TCL_ERROR;
 		    goto done;
 		}
 	    }
-	    result = ConfigureFrame(interp, framePtr, argc-2, argv+2,
+	    result = ConfigureFrame(interp, framePtr, objc-2, objv+2,
 		    TK_CONFIG_ARGV_ONLY);
 	}
-    } else {
-	Tcl_AppendResult(interp, "bad option \"", argv[1],
-		"\": must be cget or configure", (char *) NULL);
-	result = TCL_ERROR;
+	break;
+      }
     }
 
     done:
@@ -648,7 +650,7 @@ TileChangedProc(clientData, tile, itemPtr)
  *
  * ConfigureFrame --
  *
- *	This procedure is called to process an argv/argc list, plus
+ *	This procedure is called to process an objv/objc list, plus
  *	the Tk option database, in order to configure (or
  *	reconfigure) a frame widget.
  *
@@ -665,12 +667,12 @@ TileChangedProc(clientData, tile, itemPtr)
  */
 
 static int
-ConfigureFrame(interp, framePtr, argc, argv, flags)
+ConfigureFrame(interp, framePtr, objc, objv, flags)
     Tcl_Interp *interp;		/* Used for error reporting. */
     register Frame *framePtr;	/* Information about widget;  may or may
 				 * not already have values for some fields. */
-    int argc;			/* Number of valid entries in argv. */
-    char **argv;		/* Arguments. */
+    int objc;			/* Number of valid entries in objv. */
+    Tcl_Obj *CONST objv[];	/* Arguments. */
     int flags;			/* Flags to pass to Tk_ConfigureWidget. */
 {
     char *oldMenuName;
@@ -690,7 +692,8 @@ ConfigureFrame(interp, framePtr, argc, argv, flags)
     }
     
     if (Tk_ConfigureWidget(interp, framePtr->tkwin, configSpecs,
-	    argc, argv, (char *) framePtr, flags | framePtr->mask) != TCL_OK) {
+	    objc, (char **) objv, (char *) framePtr,
+	    flags | framePtr->mask | TK_CONFIG_OBJS) != TCL_OK) {
 	return TCL_ERROR;
     }
 
@@ -765,7 +768,6 @@ DisplayFrame(clientData)
 {
     register Frame *framePtr = (Frame *) clientData;
     register Tk_Window tkwin = framePtr->tkwin;
-    GC gc;
     void (* drawFunction) _ANSI_ARGS_((Tk_Window, Drawable, Tk_3DBorder,
 	    int, int, int, int, int, int)) = Tk_Fill3DRectangle;
 
@@ -827,15 +829,19 @@ DisplayFrame(clientData)
 		framePtr->borderWidth, framePtr->relief);
     }
     if (framePtr->highlightWidth != 0) {
+        GC fgGC, bgGC;
+        
+	bgGC = Tk_GCForColor(framePtr->highlightBgColorPtr,
+		Tk_WindowId(tkwin));
 	if (framePtr->flags & GOT_FOCUS) {
-	    gc = Tk_GCForColor(framePtr->highlightColorPtr,
+	    fgGC = Tk_GCForColor(framePtr->highlightColorPtr,
+		    Tk_WindowId(tkwin));
+	    TkpDrawHighlightBorder(tkwin, fgGC, bgGC, framePtr->highlightWidth,
 		    Tk_WindowId(tkwin));
 	} else {
-	    gc = Tk_GCForColor(framePtr->highlightBgColorPtr,
+	    TkpDrawHighlightBorder(tkwin, bgGC, bgGC, framePtr->highlightWidth,
 		    Tk_WindowId(tkwin));
 	}
-	Tk_DrawFocusHighlight(tkwin, gc, framePtr->highlightWidth,
-		Tk_WindowId(tkwin));
     }
 }
 
