@@ -8,12 +8,12 @@
  *	for Tk applications.
  *
  * Copyright (c) 1990-1994 The Regents of the University of California.
- * Copyright (c) 1994-1996 Sun Microsystems, Inc.
+ * Copyright (c) 1994-1997 Sun Microsystems, Inc.
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkMain.c,v 1.2 1998/09/14 18:23:14 stanton Exp $
+ * RCS: @(#) $Id: tkMain.c,v 1.5 1999/04/28 18:18:06 redman Exp $
  */
 
 #include <ctype.h>
@@ -21,17 +21,29 @@
 #include <string.h>
 #include <tcl.h>
 #include <tk.h>
+#include "tkInt.h"
 #ifdef NO_STDLIB_H
 #   include "../compat/stdlib.h"
 #else
 #   include <stdlib.h>
 #endif
-
-#ifndef TCL_USE_STUBS
-#define Tcl_Required(interp, version, exact) \
-	Tcl_PkgRequire(interp, "Tcl", version, exact)
+#ifdef __WIN32__
+#include "tkWinInt.h"
 #endif
 
+
+typedef struct ThreadSpecificData {
+    Tcl_Interp *interp;         /* Interpreter for this thread. */
+    Tcl_DString command;        /* Used to assemble lines of terminal input
+				 * into Tcl commands. */
+    Tcl_DString line;           /* Used to read the next line from the
+				 * terminal input. */
+    int tty;                    /* Non-zero means standard input is a 
+				 * terminal-like device.  Zero means it's
+				 * a file. */
+} ThreadSpecificData;
+Tcl_ThreadDataKey dataKey;
+
 /*
  * Declarations for various library procedures and variables (don't want
  * to include tkInt.h or tkPort.h here, because people might copy this
@@ -41,26 +53,12 @@
  * some systems.
  */
 
-extern int		isatty _ANSI_ARGS_((int fd));
 #if !defined(__WIN32__) && !defined(_WIN32)
+extern int		isatty _ANSI_ARGS_((int fd));
 extern char *		strrchr _ANSI_ARGS_((CONST char *string, int c));
 #endif
-extern void		TkpDisplayWarning _ANSI_ARGS_((char *msg,
-			    char *title));
-void			TkConsoleCreate_ _ANSI_ARGS_((void));
-
-/*
- * Global variables used by the main program:
- */
-
-static Tcl_Interp *interp;	/* Interpreter for this application. */
-static Tcl_DString command;	/* Used to assemble lines of terminal input
-				 * into Tcl commands. */
-static Tcl_DString line;	/* Used to read the next line from the
-                                 * terminal input. */
-static int tty;			/* Non-zero means standard input is a
-				 * terminal-like device.  Zero means it's
-				 * a file. */
+extern void		TkpDisplayWarning _ANSI_ARGS_((CONST char *msg,
+			    CONST char *title));
 
 /*
  * Forward declarations for procedures defined later in this file.
@@ -73,7 +71,7 @@ static void		StdinProc _ANSI_ARGS_((ClientData clientData,
 /*
  *----------------------------------------------------------------------
  *
- * Tk_Main, TkMain --
+ * TkMainEx --
  *
  *	Main program for Wish and most other Tk-based applications.
  *
@@ -88,50 +86,46 @@ static void		StdinProc _ANSI_ARGS_((ClientData clientData,
  *
  *----------------------------------------------------------------------
  */
-
-#undef Tk_Main
-
 void
-Tk_Main(argc, argv, appInitProc)
+Tk_MainEx(argc, argv, appInitProc, interp)
     int argc;				/* Number of arguments. */
     char **argv;			/* Array of argument strings. */
     Tcl_AppInitProc *appInitProc;	/* Application-specific initialization
 					 * procedure to call after most
 					 * initialization but before starting
 					 * to execute commands. */
-{
-    TkMain(argc, argv, appInitProc, Tcl_CreateInterp());
-    return;
-}
-
-void
-TkMain(argc, argv, appInitProc, iPtr)
-    int argc;				/* Number of arguments. */
-    char **argv;			/* Array of argument strings. */
-    Tcl_AppInitProc *appInitProc;	/* Application-specific initialization
-					 * procedure to call after most
-					 * initialization but before starting
-					 * to execute commands. */
-    Tcl_Interp *iPtr;
+    Tcl_Interp *interp;
 {
     char *args, *fileName;
-    char buf[20];
+    char buf[TCL_INTEGER_SPACE];
     int code;
     size_t length;
     Tcl_Channel inChannel, outChannel;
+    Tcl_DString argString;
+    ThreadSpecificData *tsdPtr;
+#ifdef __WIN32__
+    HANDLE handle;
+#endif
 
-    if (Tcl_Required(iPtr, TCL_VERSION, 0) == NULL) {
-	TkpDisplayWarning(iPtr->result, "Application initialization failed");
-	exit(0);
+    /*
+     * Ensure that we are getting the matching version of Tcl.  This is
+     * really only an issue when Tk is loaded dynamically.
+     */
+
+    if (Tcl_InitStubs(interp, TCL_VERSION, 1) == NULL) {
+	abort();
     }
 
-    Tcl_Required(iPtr, TCL_VERSION, 0);
-#if defined(MAC_TCL) || defined(__WIN32__)
-    TkConsoleCreate_();
-#endif
+    tsdPtr = (ThreadSpecificData *) 
+	Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
+    
     Tcl_FindExecutable(argv[0]);
-    interp = iPtr;
+    tsdPtr->interp = interp;
 
+#if (defined(__WIN32__) || defined(MAC_TCL))
+    Tk_InitConsoleChannels(interp);
+#endif
+    
 #ifdef TCL_MEM_DEBUG
     Tcl_InitMemory(interp);
 #endif
@@ -163,12 +157,19 @@ TkMain(argc, argv, appInitProc, iPtr)
      */
 
     args = Tcl_Merge(argc-1, argv+1);
-    Tcl_SetVar(interp, "argv", args, TCL_GLOBAL_ONLY);
+    Tcl_ExternalToUtfDString(NULL, args, -1, &argString);
+    Tcl_SetVar(interp, "argv", Tcl_DStringValue(&argString), TCL_GLOBAL_ONLY);
+    Tcl_DStringFree(&argString);
     ckfree(args);
     sprintf(buf, "%d", argc-1);
+
+    if (fileName == NULL) {
+	Tcl_ExternalToUtfDString(NULL, argv[0], -1, &argString);
+    } else {
+	fileName = Tcl_ExternalToUtfDString(NULL, fileName, -1, &argString);
+    }
     Tcl_SetVar(interp, "argc", buf, TCL_GLOBAL_ONLY);
-    Tcl_SetVar(interp, "argv0", (fileName != NULL) ? fileName : argv[0],
-	    TCL_GLOBAL_ONLY);
+    Tcl_SetVar(interp, "argv0", Tcl_DStringValue(&argString), TCL_GLOBAL_ONLY);
 
     /*
      * Set the "tcl_interactive" variable.
@@ -182,19 +183,39 @@ TkMain(argc, argv, appInitProc, iPtr)
      */
 
 #ifdef __WIN32__
-    tty = 1;
+    handle = GetStdHandle(STD_INPUT_HANDLE);
+
+    if ((handle == INVALID_HANDLE_VALUE) || (handle == 0) 
+	     || (GetFileType(handle) == FILE_TYPE_UNKNOWN)) {
+	/*
+	 * If it's a bad or closed handle, then it's been connected
+	 * to a wish console window.
+	 */
+
+	tsdPtr->tty = 1;
+    } else if (GetFileType(handle) == FILE_TYPE_CHAR) {
+	/*
+	 * A character file handle is a tty by definition.
+	 */
+
+	tsdPtr->tty = 1;
+    } else {
+	tsdPtr->tty = 0;
+    }
+
 #else
-    tty = isatty(0);
+    tsdPtr->tty = isatty(0);
 #endif
     Tcl_SetVar(interp, "tcl_interactive",
-	    ((fileName == NULL) && tty) ? "1" : "0", TCL_GLOBAL_ONLY);
+	    ((fileName == NULL) && tsdPtr->tty) ? "1" : "0", TCL_GLOBAL_ONLY);
 
     /*
      * Invoke application-specific initialization.
      */
 
     if ((*appInitProc)(interp) != TCL_OK) {
-	TkpDisplayWarning(Tcl_GetStringResult(interp), "Application initialization failed");
+	TkpDisplayWarning(Tcl_GetStringResult(interp),
+		"Application initialization failed");
     }
 
     /*
@@ -202,6 +223,7 @@ TkMain(argc, argv, appInitProc, iPtr)
      */
 
     if (fileName != NULL) {
+	Tcl_ResetResult(interp);
 	code = Tcl_EvalFile(interp, fileName);
 	if (code != TCL_OK) {
 	    /*
@@ -215,7 +237,7 @@ TkMain(argc, argv, appInitProc, iPtr)
 	    Tcl_DeleteInterp(interp);
 	    Tcl_Exit(1);
 	}
-	tty = 0;
+	tsdPtr->tty = 0;
     } else {
 
 	/*
@@ -233,17 +255,18 @@ TkMain(argc, argv, appInitProc, iPtr)
 	    Tcl_CreateChannelHandler(inChannel, TCL_READABLE, StdinProc,
 		    (ClientData) inChannel);
 	}
-	if (tty) {
+	if (tsdPtr->tty) {
 	    Prompt(interp, 0);
 	}
     }
+    Tcl_DStringFree(&argString);
 
     outChannel = Tcl_GetStdChannel(TCL_STDOUT);
     if (outChannel) {
 	Tcl_Flush(outChannel);
     }
-    Tcl_DStringInit(&command);
-    Tcl_DStringInit(&line);
+    Tcl_DStringInit(&tsdPtr->command);
+    Tcl_DStringInit(&tsdPtr->line);
     Tcl_ResetResult(interp);
 
     /*
@@ -286,12 +309,15 @@ StdinProc(clientData, mask)
     char *cmd;
     int code, count;
     Tcl_Channel chan = (Tcl_Channel) clientData;
+    ThreadSpecificData *tsdPtr = (ThreadSpecificData *) 
+            Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
+    Tcl_Interp *interp = tsdPtr->interp;
 
-    count = Tcl_Gets(chan, &line);
+    count = Tcl_Gets(chan, &tsdPtr->line);
 
     if (count < 0) {
 	if (!gotPartial) {
-	    if (tty) {
+	    if (tsdPtr->tty) {
 		Tcl_Exit(0);
 	    } else {
 		Tcl_DeleteChannelHandler(chan, StdinProc, (ClientData) chan);
@@ -300,9 +326,10 @@ StdinProc(clientData, mask)
 	} 
     }
 
-    (void) Tcl_DStringAppend(&command, Tcl_DStringValue(&line), -1);
-    cmd = Tcl_DStringAppend(&command, "\n", -1);
-    Tcl_DStringFree(&line);
+    (void) Tcl_DStringAppend(&tsdPtr->command, Tcl_DStringValue(
+            &tsdPtr->line), -1);
+    cmd = Tcl_DStringAppend(&tsdPtr->command, "\n", -1);
+    Tcl_DStringFree(&tsdPtr->line);
     if (!Tcl_CommandComplete(cmd)) {
         gotPartial = 1;
         goto prompt;
@@ -325,17 +352,14 @@ StdinProc(clientData, mask)
 	Tcl_CreateChannelHandler(chan, TCL_READABLE, StdinProc,
 		(ClientData) chan);
     }
-    Tcl_DStringFree(&command);
-    if (*Tcl_GetStringResult(interp) != 0) {
-	if ((code != TCL_OK) || (tty)) {
-	    /*
-	     * The statement below used to call "printf", but that resulted
-	     * in core dumps under Solaris 2.3 if the result was very long.
-             *
-             * NOTE: This probably will not work under Windows either.
-	     */
-
-	    puts(Tcl_GetStringResult(interp));
+    Tcl_DStringFree(&tsdPtr->command);
+    if (Tcl_GetStringResult(interp)[0] != '\0') {
+	if ((code != TCL_OK) || (tsdPtr->tty)) {
+	    chan = Tcl_GetStdChannel(TCL_STDOUT);
+	    if (chan) {
+		Tcl_WriteObj(chan, Tcl_GetObjResult(interp));
+		Tcl_WriteChars(chan, "\n", 1);
+	    }
 	}
     }
 
@@ -344,7 +368,7 @@ StdinProc(clientData, mask)
      */
 
     prompt:
-    if (tty) {
+    if (tsdPtr->tty) {
 	Prompt(interp, gotPartial);
     }
     Tcl_ResetResult(interp);
@@ -382,8 +406,8 @@ Prompt(interp, partial)
     promptCmd = Tcl_GetVar(interp,
 	partial ? "tcl_prompt2" : "tcl_prompt1", TCL_GLOBAL_ONLY);
     if (promptCmd == NULL) {
-	outChannel = Tcl_GetChannel(interp, "stdout", NULL);
 defaultPrompt:
+	outChannel = Tcl_GetChannel(interp, "stdout", NULL);
 	if (!partial) {
 
             /*
@@ -393,7 +417,7 @@ defaultPrompt:
              */
 
             if (outChannel != (Tcl_Channel) NULL) {
-                Tcl_Write(outChannel, "% ", 2);
+                Tcl_WriteChars(outChannel, "% ", 2);
             }
 	}
     } else {
@@ -410,14 +434,15 @@ defaultPrompt:
             
 	    errChannel = Tcl_GetChannel(interp, "stderr", NULL);
             if (errChannel != (Tcl_Channel) NULL) {
-                Tcl_Write(errChannel, Tcl_GetStringResult(interp), -1);
-                Tcl_Write(errChannel, "\n", 1);
+                Tcl_WriteObj(errChannel, Tcl_GetObjResult(interp));
+                Tcl_WriteChars(errChannel, "\n", 1);
             }
 	    goto defaultPrompt;
 	} else if (*Tcl_GetStringResult(interp) && outChannel) {
 	    Tcl_Write(outChannel, Tcl_GetStringResult(interp), -1);
 	}
     }
+    outChannel = Tcl_GetChannel(interp, "stdout", NULL);
     if (outChannel != (Tcl_Channel) NULL) {
         Tcl_Flush(outChannel);
     }
