@@ -217,7 +217,7 @@ static void		OpenIM _ANSI_ARGS_((TkDisplay *dispPtr));
 static void		UnlinkWindow _ANSI_ARGS_((TkWindow *winPtr));
 #ifdef TK_USE_INPUT_METHODS
 #ifdef I18N_IMPROVE
-static Bool		CanInitiateIm _ANSI_ARGS_((char *locale));
+static int		CanInitiateIm _ANSI_ARGS_((Tcl_Obj *locale));
 static void		IMInstantiateCallback _ANSI_ARGS_ ((Display *display,
 				XPointer clientData, XPointer callData));
 static void		IMDestroyCallback _ANSI_ARGS_ ((XIM im, XPointer clientData, 
@@ -2479,13 +2479,14 @@ Tk_StrictMotif(tkwin)
 #ifdef TK_USE_INPUT_METHODS
 #ifdef I18N_IMPROVE
 
-static Bool
-CanInitiateIm(locale)
-     char *locale;
+static int
+CanInitiateIm(loc)
+     Tcl_Obj *loc;
 {
     static int inited = 0;
-    static Bool ret = False;
-    
+    static int ret = 0;
+    char *locale = Tcl_GetString(loc);
+
     if (inited == 0) {
 	char *curSpec = setlocale(LC_ALL, NULL);
 
@@ -2665,16 +2666,15 @@ OpenIM(dispPtr)
 
 #ifdef I18N_IMPROVE
     int styleFound = 0;
-    char *imLocale = NULL;
-    char *imEncoding = NULL;
+    Tcl_Obj *imLocale = NULL;
+    Tcl_Obj *imEncoding = NULL;
 
     dispPtr->inputMethod = NULL;
     dispPtr->imEncoding = NULL;
     dispPtr->lastFocusedIC = NULL;
 
     TclpGetStartupLocaleEncoding(&imEncoding, &imLocale);
-    if (imEncoding == NULL || imEncoding[0] == '\0' ||
-	imLocale == NULL || imLocale[0] == '\0') {
+    if (imEncoding == NULL || imLocale == NULL) {
 	return;
     }
     if (CanInitiateIm(imLocale) == False) {
@@ -2722,7 +2722,7 @@ OpenIM(dispPtr)
 	/*
 	 * Create a Tcl_Encoding for XmbLookupString() conversion.
 	 */
-	dispPtr->imEncoding = Tcl_GetEncoding(NULL, imEncoding);
+	dispPtr->imEncoding = Tcl_GetEncoding(NULL, Tcl_GetString(imEncoding));
 #ifdef XNDestroyCallback
 	dispPtr->destroyCallback.client_data = (XPointer)dispPtr;
 	dispPtr->destroyCallback.callback = (XIMProc)IMDestroyCallback;
@@ -2972,7 +2972,6 @@ Initialize(interp)
     Tcl_Obj *args[20];
     Tcl_DString class;
     ThreadSpecificData *tsdPtr;
-    static char version[] = TK_PATCH_LEVEL;
     
 #if defined(__WIN32__) && defined(STATIC_BUILD) && !defined(NO_STATIC_INIT)
     /*
@@ -3214,7 +3213,7 @@ Initialize(interp)
     }
     Tcl_MutexUnlock(&windowMutex);
 
-    if (Tcl_PkgRequire(interp, "Tcl", TCL_VERSION, 1) == NULL) {
+    if (Tcl_PkgRequire(interp, "Tcl", TCL_VERSION, 0) == NULL) {
 	code = TCL_ERROR;
 	goto done;
     }
@@ -3223,8 +3222,7 @@ Initialize(interp)
      * Provide Tk and its stub table.
      */
 
-    version[5] = 0;
-    code = Tcl_PkgProvideEx(interp, "Tk", version, (ClientData) &tkStubs);
+    code = Tcl_PkgProvideEx(interp, "Tk", TK_VERSION, (ClientData) &tkStubs);
     if (code != TCL_OK) {
 	goto done;
     }
