@@ -3,7 +3,7 @@
 # This file defines the default bindings for Tk entry widgets and provides
 # procedures that help in implementing those bindings.
 #
-# RCS: @(#) $Id: entry.tcl,v 1.5 1998/09/14 18:23:23 stanton Exp $
+# RCS: @(#) $Id: entry.tcl,v 1.7 1999/08/09 16:52:06 hobbs Exp $
 #
 # Copyright (c) 1992-1994 The Regents of the University of California.
 # Copyright (c) 1994-1997 Sun Microsystems, Inc.
@@ -26,30 +26,31 @@
 #			char, word, or line.
 # x, y -		Last known mouse coordinates for scanning
 #			and auto-scanning.
+# data -		Used for Cut and Copy
 #-------------------------------------------------------------------------
 
 #-------------------------------------------------------------------------
 # The code below creates the default class bindings for entries.
 #-------------------------------------------------------------------------
 bind Entry <<Cut>> {
-    if {![catch {set data [string range [%W get] [%W index sel.first]\
-	    [expr {[%W index sel.last] - 1}]]}]} {
+    if {![catch {tkEntryGetSelection %W} tkPriv(data)]} {
 	clipboard clear -displayof %W
-	clipboard append -displayof %W $data
+	clipboard append -displayof %W $tkPriv(data)
 	%W delete sel.first sel.last
+	unset tkPriv(data)
     }
 }
 bind Entry <<Copy>> {
-    if {![catch {set data [string range [%W get] [%W index sel.first]\
-	    [expr {[%W index sel.last] - 1}]]}]} {
+    if {![catch {tkEntryGetSelection %W} tkPriv(data)]} {
 	clipboard clear -displayof %W
-	clipboard append -displayof %W $data
+	clipboard append -displayof %W $tkPriv(data)
+	unset tkPriv(data)
     }
 }
 bind Entry <<Paste>> {
     global tcl_platform
     catch {
-	if {[string compare $tcl_platform(platform) unix]} {
+      if {[string compare $tcl_platform(platform) "unix"]} {
 	    catch {
 		%W delete sel.first sel.last
 	    }
@@ -201,18 +202,25 @@ bind Entry <Escape> {# nothing}
 bind Entry <Return> {# nothing}
 bind Entry <KP_Enter> {# nothing}
 bind Entry <Tab> {# nothing}
-if {![string compare $tcl_platform(platform) macintosh]} {
+if {![string compare $tcl_platform(platform) "macintosh"]} {
 	bind Entry <Command-KeyPress> {# nothing}
 }
 
 # On Windows, paste is done using Shift-Insert.  Shift-Insert already
 # generates the <<Paste>> event, so we don't need to do anything here.
-if {[string compare $tcl_platform(platform) windows]} {
+if {[string compare $tcl_platform(platform) "windows"]} {
     bind Entry <Insert> {
 	catch {tkEntryInsert %W [selection get -displayof %W]}
     }
 }
 
+# input method configuration binding (unix only)
+if {![string compare $tcl_platform(platform) "unix"]} {
+    if {[string compare [info commands imconfigure] {}]} {
+	bind Text <FocusIn> {catch {imconfigure %W}}
+    }
+}
+
 # Additional emacs-like bindings:
 
 bind Entry <Control-a> {
@@ -335,7 +343,7 @@ proc tkEntryButton1 {w x} {
     set tkPriv(pressX) $x
     $w icursor [tkEntryClosestGap $w $x]
     $w selection from insert
-    if {![string compare [$w cget -state] normal]} {focus $w}
+    if {![string compare [$w cget -state] "normal"]} {focus $w}
 }
 
 # tkEntryMouseSelect --
@@ -405,7 +413,7 @@ proc tkEntryPaste {w x} {
 
     $w icursor [tkEntryClosestGap $w $x]
     catch {$w insert insert [selection get -displayof $w]}
-    if {![string compare [$w cget -state] normal]} {focus $w}
+    if {![string compare [$w cget -state] "normal"]} {focus $w}
 }
 
 # tkEntryAutoScan --
@@ -462,7 +470,7 @@ proc tkEntryKeySelect {w new} {
 # s -		The string to insert (usually just a single character)
 
 proc tkEntryInsert {w s} {
-    if {![string compare $s {}]} {
+    if {![string compare $s ""]} {
 	return
     }
     catch {
@@ -662,7 +670,7 @@ proc tkEntryTranspose w {
 # w -		The entry window in which the cursor is to move.
 # start -	Position at which to start search.
 
-if {![string compare $tcl_platform(platform) windows]}  {
+if {![string compare $tcl_platform(platform) "windows"]}  {
     proc tkEntryNextWord {w start} {
 	set pos [tcl_endOfWord [$w get] [$w index $start]]
 	if {$pos >= 0} {
@@ -699,4 +707,18 @@ proc tkEntryPreviousWord {w start} {
     }
     return $pos
 }
+# tkEntryGetSelection --
+#
+# Returns the selected text of the entry with respect to the -show option.
+#
+# Arguments:
+# w -         The entry window from which the text to get
 
+proc tkEntryGetSelection {w} {
+    set entryString [string range [$w get] [$w index sel.first] \
+                       [expr [$w index sel.last] - 1]]
+    if {[$w cget -show] != ""} {
+      regsub -all . $entryString [string index [$w cget -show] 0] entryString
+    }
+    return $entryString
+}
