@@ -3,7 +3,7 @@
 # Initialization script normally executed in the interpreter for each
 # Tk-based application.  Arranges class bindings for widgets.
 #
-# RCS: @(#) $Id: tk.tcl,v 1.7 1999/04/16 01:51:27 stanton Exp $
+# RCS: @(#) $Id: tk.tcl,v 1.10 1999/08/13 02:58:17 hobbs Exp $
 #
 # Copyright (c) 1992-1994 The Regents of the University of California.
 # Copyright (c) 1994-1996 Sun Microsystems, Inc.
@@ -14,13 +14,13 @@
 
 # Insist on running with compatible versions of Tcl and Tk.
 
-package require -exact Tk 8.1
-package require -exact Tcl 8.1
+package require Tcl 8.2
+package require Tk 8.2
 
 # Add Tk's directory to the end of the auto-load search path, if it
 # isn't already on the path:
 
-if {[info exists auto_path] && [string compare $tk_library {}] &&
+if {[info exists auto_path] && [string compare {} $tk_library] && \
 	[lsearch -exact $auto_path $tk_library] < 0} {
     lappend auto_path $tk_library
 }
@@ -180,7 +180,8 @@ switch $tcl_platform(platform) {
 # Read in files that define all of the class bindings.
 # ----------------------------------------------------------------------
 
-if {[string compare $tk_library {}]} {
+if {[string compare $tcl_platform(platform) "macintosh"] &&
+    [string compare {} $tk_library]} {
     source [file join $tk_library button.tcl]
     source [file join $tk_library entry.tcl]
     source [file join $tk_library listbox.tcl]
@@ -197,18 +198,6 @@ if {[string compare $tk_library {}]} {
 bind all <Tab> {tkTabToWindow [tk_focusNext %W]}
 bind all <Shift-Tab> {tkTabToWindow [tk_focusPrev %W]}
 
-# ----------------------------------------------------------------------
-# Input method configuration binding (unix only)
-# ----------------------------------------------------------------------
-
-if {[string compare $tcl_platform(platform) "unix"] == 0} {
-    if {[info commands imconfigure] != {}} {
-	foreach i "Text Entry" {
-	    bind $i <FocusIn> {catch {imconfigure %W}}
-	}
-    }
-}
-
 # tkCancelRepeat --
 # This procedure is invoked to cancel an auto-repeat action described
 # by tkPriv(afterId).  It's used by several widgets to auto-scroll
@@ -233,8 +222,8 @@ proc tkCancelRepeat {} {
 
 proc tkTabToWindow {w} {
     if {![string compare [winfo class $w] Entry]} {
-	$w select range 0 end
-	$w icur end
+	$w selection range 0 end
+	$w icursor end
     }
     focus $w
 }
