@@ -3,12 +3,13 @@
  *
  *	Main entry point for wish and other Tk-based applications.
  *
- * Copyright (c) 1995 Sun Microsystems, Inc.
+ * Copyright (c) 1995-1997 Sun Microsystems, Inc.
+ * Copyright (c) 1998-1999 by Scriptics Corporation.
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: winMain.c,v 1.3 1998/09/14 18:24:02 stanton Exp $
+ * RCS: @(#) $Id: winMain.c,v 1.7 1999/04/28 18:18:07 redman Exp $
  */
 
 #undef TCL_USE_STUBS
@@ -20,29 +21,32 @@
 #include <malloc.h>
 #include <locale.h>
 
+#include "tkInt.h"
+
 /*
  * The following declarations refer to internal Tk routines.  These
  * interfaces are available for use, but are not supported.
  */
 
-EXTERN void		TkConsoleCreate();
-EXTERN int		TkConsoleInit(Tcl_Interp *interp);
-EXTERN void		TclWinInit(HINSTANCE hInstance);
-EXTERN void		TkWinXInit(HINSTANCE hInstance);
-EXTERN void		TkWinXCleanup(HINSTANCE hInstance);
 
 /*
  * Forward declarations for procedures defined later in this file:
  */
 
 static void		setargv _ANSI_ARGS_((int *argcPtr, char ***argvPtr));
-extern int		Registry_Init _ANSI_ARGS_((Tcl_Interp *interp));
-static void		WishPanic _ANSI_ARGS_(TCL_VARARGS(char *,format));
+static void		WishPanic _ANSI_ARGS_(TCL_VARARGS(CONST char *,format));
 
 #ifdef TK_TEST
-EXTERN int		Tktest_Init(Tcl_Interp *interp);
+extern int		Tktest_Init(Tcl_Interp *interp);
 #endif /* TK_TEST */
 
+#ifdef TCL_TEST
+extern int		Objtest_Init _ANSI_ARGS_((Tcl_Interp *interp));
+extern int		Tcltest_Init _ANSI_ARGS_((Tcl_Interp *interp));
+#endif /* TCL_TEST */
+
+static BOOL consoleRequired = TRUE;
+
 
 /*
  *----------------------------------------------------------------------
@@ -68,9 +72,8 @@ WinMain(hInstance, hPrevInstance, lpszCmdLine, nCmdShow)
     LPSTR lpszCmdLine;
     int nCmdShow;
 {
-    char **argv, *p;
+    char **argv;
     int argc;
-    char buffer[MAX_PATH];
 
     Tcl_SetPanicProc(WishPanic);
 
@@ -80,7 +83,7 @@ WinMain(hInstance, hPrevInstance, lpszCmdLine, nCmdShow)
      */
 
     setlocale(LC_ALL, "C");
-
+    setargv(&argc, &argv);
 
     /*
      * Increase the application queue size from default value of 8.
@@ -89,36 +92,16 @@ WinMain(hInstance, hPrevInstance, lpszCmdLine, nCmdShow)
      * This is only needed for Windows 3.x, since NT dynamically expands
      * the queue.
      */
+
     SetMessageQueue(64);
 
     /*
      * Create the console channels and install them as the standard
-     * channels.  All I/O will be discarded until TkConsoleInit is
+     * channels.  All I/O will be discarded until Tk_CreateConsoleWindow is
      * called to attach the console to a text widget.
      */
 
-    TkConsoleCreate();
-
-    setargv(&argc, &argv);
-
-    /*
-     * Replace argv[0] with full pathname of executable, and forward
-     * slashes substituted for backslashes.
-     */
-
-    GetModuleFileName(NULL, buffer, sizeof(buffer));
-    argv[0] = buffer;
-    for (p = buffer; *p != '\0'; p++) {
-	if (*p == '\\') {
-	    *p = '/';
-	}
-    }
-
-#ifdef Tk_Init
-    TclWinInit(hInstance);
-    TkWinXInit(hInstance);
-    Tcl_CreateExitHandler((Tcl_ExitProc *) TkWinXCleanup, (ClientData) hInstance);
-#endif
+    consoleRequired = TRUE;
 
     Tk_Main(argc, argv, Tcl_AppInit);
     return 1;
@@ -136,7 +119,7 @@ WinMain(hInstance, hPrevInstance, lpszCmdLine, nCmdShow)
  *
  * Results:
  *	Returns a standard Tcl completion code, and leaves an error
- *	message in interp->result if an error occurs.
+ *	message in the interp's result if an error occurs.
  *
  * Side effects:
  *	Depends on the startup script.
@@ -151,10 +134,6 @@ Tcl_AppInit(interp)
     if (Tcl_Init(interp) == TCL_ERROR) {
 	goto error;
     }
-#ifdef Tcl_Init
-    Tcl_StaticPackage((Tcl_Interp *) NULL, "registry", Registry_Init,
-            (Tcl_PackageInitProc *) NULL);
-#endif
     if (Tk_Init(interp) == TCL_ERROR) {
 	goto error;
     }
@@ -165,10 +144,25 @@ Tcl_AppInit(interp)
      * application.
      */
 
-    if (TkConsoleInit(interp) == TCL_ERROR) {
-	goto error;
+    if (consoleRequired) {
+	if (Tk_CreateConsoleWindow(interp) == TCL_ERROR) {
+	    goto error;
+	}
     }
 
+#ifdef TCL_TEST
+    if (Tcltest_Init(interp) == TCL_ERROR) {
+	return TCL_ERROR;
+    }
+    Tcl_StaticPackage(interp, "Tcltest", Tcltest_Init,
+            (Tcl_PackageInitProc *) NULL);
+    if (Objtest_Init(interp) == TCL_ERROR) {
+	return TCL_ERROR;
+    }
+    Tcl_StaticPackage(interp, "Objtest", Objtest_Init,
+            (Tcl_PackageInitProc *) NULL);
+#endif /* TCL_TEST */
+
 #ifdef TK_TEST
     if (Tktest_Init(interp) == TCL_ERROR) {
 	goto error;
@@ -181,7 +175,7 @@ Tcl_AppInit(interp)
     return TCL_OK;
 
 error:
-    WishPanic(interp->result);
+    WishPanic(Tcl_GetStringResult(interp));
     return TCL_ERROR;
 }
 
@@ -202,13 +196,13 @@ error:
  */
 
 void
-WishPanic TCL_VARARGS_DEF(char *,arg1)
+WishPanic TCL_VARARGS_DEF(CONST char *,arg1)
 {
     va_list argList;
     char buf[1024];
-    char *format;
-    
-    format = TCL_VARARGS_START(char *,arg1,argList);
+    CONST char *format;
+
+    format = TCL_VARARGS_START(CONST char *,arg1,argList);
     vsprintf(buf, format, argList);
 
     MessageBeep(MB_ICONEXCLAMATION);
@@ -255,7 +249,7 @@ setargv(argcPtr, argvPtr)
     char **argv;
     int argc, size, inquote, copy, slashes;
     
-    cmdLine = GetCommandLine();
+    cmdLine = GetCommandLine();	/* INTL: BUG */
 
     /*
      * Precompute an overly pessimistic guess at the number of arguments
@@ -264,9 +258,9 @@ setargv(argcPtr, argvPtr)
 
     size = 2;
     for (p = cmdLine; *p != '\0'; p++) {
-	if (isspace(*p)) {
+	if ((*p == ' ') || (*p == '\t')) {	/* INTL: ISO space. */
 	    size++;
-	    while (isspace(*p)) {
+	    while ((*p == ' ') || (*p == '\t')) { /* INTL: ISO space. */
 		p++;
 	    }
 	    if (*p == '\0') {
@@ -274,8 +268,8 @@ setargv(argcPtr, argvPtr)
 	    }
 	}
     }
-    argSpace = (char *) ckalloc((unsigned) (size * sizeof(char *) 
-	    + strlen(cmdLine) + 1));
+    argSpace = (char *) Tcl_Alloc(
+	    (unsigned) (size * sizeof(char *) + strlen(cmdLine) + 1));
     argv = (char **) argSpace;
     argSpace += size * sizeof(char *);
     size--;
@@ -283,7 +277,7 @@ setargv(argcPtr, argvPtr)
     p = cmdLine;
     for (argc = 0; argc < size; argc++) {
 	argv[argc] = arg = argSpace;
-	while (isspace(*p)) {
+	while ((*p == ' ') || (*p == '\t')) {	/* INTL: ISO space. */
 	    p++;
 	}
 	if (*p == '\0') {
@@ -317,7 +311,8 @@ setargv(argcPtr, argvPtr)
 		slashes--;
 	    }
 
-	    if ((*p == '\0') || (!inquote && isspace(*p))) {
+	    if ((*p == '\0')
+		    || (!inquote && ((*p == ' ') || (*p == '\t')))) { /* INTL: ISO space. */
 		break;
 	    }
 	    if (copy != 0) {
@@ -335,3 +330,53 @@ setargv(argcPtr, argvPtr)
     *argvPtr = argv;
 }
 
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * main --
+ *
+ *	Main entry point from the console.
+ *
+ * Results:
+ *	None: Tk_Main never returns here, so this procedure never
+ *      returns either.
+ *
+ * Side effects:
+ *	Whatever the applications does.
+ *
+ *----------------------------------------------------------------------
+ */
+
+int main(int argc, char **argv)
+{
+    Tcl_SetPanicProc(WishPanic);
+
+    /*
+     * Set up the default locale to be standard "C" locale so parsing
+     * is performed correctly.
+     */
+
+    setlocale(LC_ALL, "C");
+    /*
+     * Increase the application queue size from default value of 8.
+     * At the default value, cross application SendMessage of WM_KILLFOCUS
+     * will fail because the handler will not be able to do a PostMessage!
+     * This is only needed for Windows 3.x, since NT dynamically expands
+     * the queue.
+     */
+
+    SetMessageQueue(64);
+
+    /*
+     * Create the console channels and install them as the standard
+     * channels.  All I/O will be discarded until Tk_CreateConsoleWindow is
+     * called to attach the console to a text widget.
+     */
+
+    consoleRequired = FALSE;
+
+    Tk_Main(argc, argv, Tcl_AppInit);
+    return 0;
+}
+
